#include <nspireio/nspireio.h>
#include <stdlib.h>

#include "esp8266_avec_dweet.io.h"

int main()
{
  char input[5000] = {0};
  int ret;
  char cmd[300];
  char **networks = NULL;
  int nbNetworks;
  int accesPoint = -1;
  char pt_access[50];

  /* Commande basique de test d'une commande AT */
  ret = send_AT_command("AT\r\n", 100, input);
  if (ret != 1) {
      puts("Communication impossible avec l'esp8266");
      puts("verifier le cablage puis recommencer");
      puts("Appuyer sur une touche pour terminer le traitement");
      wait_key_pressed();
      return 0;
  }

  /* Commande AT un peu plus élaborée : demande de la version du firmware de l'esp8266*/
  ret = send_AT_command("AT+GMR\r\n", 100, input);

  /* Demande de la liste des points d'accès disponibles */
  ret = send_AT_command("AT+CWLAP\r\n", 5000, input);
  if (ret == 1) {
    networks = getNetworks(&nbNetworks, input);
    for (int i=0; i<=nbNetworks-1; i++) {
      nio_printf ("%d: %s\n", i, networks[i]);
    }
  }
  while (accesPoint < 0 || accesPoint > nbNetworks-1) {
    nio_printf ("Choisir le point d'acces, taper le numero ? ");
    scanf("%d",&accesPoint);
  }
  strcpy(pt_access, networks[accesPoint]);

  /* Consultation du mode configuré actuellement: station, serveur, les deux */
  ret = send_AT_command("AT+CWMODE?\r\n", 100, input);

  /* Configuration du mode client */
  ret = send_AT_command("AT+CWMODE=1\r\n", 100, input);

  /* Connexion au point d'accès */
  int loop=1;
  char password[50]="";
  while (loop <= 3) {
    nio_printf ("mot de passe du point d'acces ? ");
    scanf("%s",password);
    sprintf (cmd, "AT+CWJAP=\"%s\",\"%s\"\r\n", pt_access, password);
    ret = send_AT_command(cmd, 500, input);
    if (ret == 1)
      break;
    else if (ret != 1 && loop >=3) {
      puts("Impossible de se connecter au point d'acces");
      puts("3 tentatives echouees");
      puts("Appuyer sur une touche pour terminer le traitement");
      wait_key_pressed();
      return 0;
    }
    loop++;
  }

  /* Demande de l'adresse IP attribuée (pour consultation) */
  ret = send_AT_command("AT+CIFSR=1\r\n", 100, input);

  /* Configuration d'un canal unique de communication, un seul suffit */
  ret = send_AT_command("AT+CIPMUX=0\r\n", 100, input);

  /* 
   ** Nous allons maintenant envoyer la date et 'heure au serveur dweet.io
  */

  /* On récupère la date et l'heure au bon format */
  time_t t = time(NULL);
  struct tm tm = *localtime(&t);
  char dt[11];
  char hr[9];
  sprintf(dt, "%02u-%02u-%4u", tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900);
  sprintf(hr, "%02d:%02d:%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);

  /* On prépare les données "utiles" de la commande POST */
  char data[100];
  char lgr_data[4];
  sprintf (data, "{\"date\":\"%s\",\"heure\":\"%s\"}", dt, hr);
  sprintf(lgr_data, "%u", strlen(data));

  /* On construit la commande POST */
  char post[500];
  char id_dweet_io[20];
  char lgr_post[4];
  nio_printf ("Identifiant dweet.io ? ");
  scanf("%s",id_dweet_io);
  strcpy(post, "POST /dweet/for/");
  strcat(post, id_dweet_io);
  strcat(post, " HTTP/1.1\r\nHost: dweet.io\r\nContent-Type: application/json\r\nConnection: close\r\nContent-Length: ");
  strcat(post, lgr_data);
  strcat(post, "\r\n\r\n");
  strcat(post, data);
  strcat(post, "\r\n\r\n");

  /* Connexion au serveur web */
  ret = send_AT_command("AT+CIPSTART=\"TCP\",\"dweet.io\",80\r\n", 100, input);

  /* Construction de la commande AT qui indique la longueur */
  sprintf(lgr_post, "%u", strlen(post));
  strcpy(cmd, "AT+CIPSEND=");
  strcat(cmd, lgr_post);
  strcat(cmd, "\r\n");

  /* On y ajoute le POST */
  strcat(cmd, post);

  /* On envoit de la commande complète */
  ret = send_AT_command(cmd, 5000, input);

  puts("Traitement termine!");
  nio_printf("consulter le resultat sur: http://dweet.io/follow/%s\n", id_dweet_io);
  puts("Appuyer sur une touche pour quitter");
  wait_key_pressed();
  return 0;
}
